/*
COPYRIGHT (C) 1989-1992, MODICON INC., IASG DIVISION, ALL RIGHTS RESERVED.
NO PART OF THIS DOCUMENT MAY BE REPRODUCED IN ANY FORM WITHOUT THE
EXPRESSED WRITTEN PERMISSION OF MODICON INC., IASG DIVISION.


program name:   PG-MBPL-500
revision:       -
version:        3.30
eco number:     16491
release date:   10/16/92

REVISION INFORMATION
10/19/92 RAE -Added Windows 3.x awareness under DOS exec_netbios.
              Fixed ncb_open for slave paths, it didn't fill in NCB_CALLNAME
              properly (did not affect operation).
10/31/90 RAE - Added OS/2 compiler directives.  There are two major
              differences between DOS and OS/2.  In DOS, exec_netbios loads
              registers ES:BX with the address of an NCB, while under OS/2,
              exec_netbios simply calls the DLL library.  In DOS, the NCB
              pool is declared globally in this file (netlib.c), while under
              OS/2, the NCB pool is allocated elsewhere in the driver.  This
              affects both the ncb_reset function, and the ncb_open function.
              Also, OS/2 does not support ncb_sa85off, but does require and
              additional function called ncb_tick().  Also fixed a bug in DOS
              version, when ncb_reset was issued, it cleared the ncbs array
              for BOTH boards.
4/12/90 RAE - Changed MAX_NCB to 64 to support two hosts (max of 64 paths)
3/5/90  RAE - Modified exec_netbios routine to check for null pointer
              interrupt vectors before invoking int86.
1/31/90 RAE - Modified ncb_receive and ncb_receive_wait to add 1 to the
              length returned to account for the modbus device address
              being added to the beginning of the message, also fixed the
              device address by using the last non-zero byte of the routing
              path instead of the first byte of the routing path.
1/31/90 RAE - Removed the length fix above.

*/
/*name netlib.c*/

/* Copyright (C) Modicon, Inc. 1989-1992,  All Rights Reserved. */

/*
The netlib.c routines are used by all the example programs.
Must be compiled Large model only 


*/
 

/*
This NETBIOS Library file contains the following procedures:
ncb_reset()
ncb_status()
ncb_open()
ncb_close()
ncb_send()
ncb_receive()
ncb_send_datagram()
ncb_receive_datagram()
ncb_cancel()
ncb_sa85off()
ncb_set_slave_login()
exec_netbios()


*/
/*
include files



*/
#include <stdio.h>
#include <dos.h>
#include <process.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include "netbios.h"
#include "netlib.h"

#ifdef __OS2__
extern void mbp_netbios(NCB *abc);
#else
extern unsigned cdecl far dv_get_version(void);
extern void cdecl far dv_begin_critical(void);
extern void cdecl far dv_end_critical(void);
extern unsigned cdecl far IsEnhancedWindows(void);
extern void cdecl far WinEnterCriticalSection(void);
extern void cdecl far WinEndCriticalSection(void);
#endif

/*
macros


*/

#define MAX_NCB 64
#define NCB_PER_BOARD 32

/*
global variables


*/
#ifndef __OS2__
NCB ncbs[MAX_NCB];      /*use for program,data paths*/
NCB ncbg;               /*use for global data transfers*/
int sw_interrupt = 0x5c;    /* s/w interrupt to use for exec_netbios() */
#endif

/*
ncb_reset()


*/

int ncb_reset(adaptno)
int adaptno;
    {
#ifndef __OS2__
    int i;
    NCB *ncbp;
#endif
    NCB dummy_ncb;

	if ((adaptno != 0) && (adaptno != 1))
		return ERR_invalid;
    dummy_ncb.NCB_LANA_NUM = (unsigned char) adaptno;
    dummy_ncb.NCB_COMMAND = RESET;
    dummy_ncb.NCB_RETCODE = ERR_bad_card_num;       /*invalidate return code*/
	 dummy_ncb.NCB_LSN = 0;
	 dummy_ncb.NCB_POST_ADDRESS = NULL;
    exec_netbios(&dummy_ncb);
#ifndef __OS2__
    for (i = 0;  i < NCB_PER_BOARD;  i++) {
        /* reset them all to unused */
        ncbp = (NCB *) &ncbs[i + (NCB_PER_BOARD * adaptno)];
        ncbp->NCB_NUM = 0;
        }
#endif
    return (dummy_ncb.NCB_RETCODE);
    }

#ifndef __OS2__
/*
ncb_sa85off()
Turn off SA85 driver (RESET to turn back on)

*/

int ncb_sa85off(adaptno)
int adaptno;
    {
    NCB dummy_ncb;

    dummy_ncb.NCB_LANA_NUM = (unsigned char) adaptno;
    dummy_ncb.NCB_COMMAND = SA85_OFF;
    dummy_ncb.NCB_RETCODE = ERR_bad_card_num;       /*invalidate return code*/
    exec_netbios(&dummy_ncb);
    return (dummy_ncb.NCB_RETCODE);
    }
#endif

/*
ncb_status()

Configuration status.
Performs a quick test of the shared ram interface.
Handy for verifying correct window address settings.

*/

int ncb_status(ncbp, adaptno)
NCB *ncbp;
int adaptno;
    {
    ncbp->NCB_LANA_NUM = (unsigned char) adaptno;
    ncbp->NCB_COMMAND = STATUS;
    ncbp->NCB_RETCODE = ERR_bad_card_num;       /*invalidate return code*/
	 ncbp->NCB_LSN = 0;
	 ncbp->NCB_POST_ADDRESS = NULL;
    exec_netbios(ncbp);
    return (ncbp->NCB_RETCODE);
    }

/*
ncb_send()


Send a message on the given path.
*/

int ncb_send(ncbp, length, buffer, timeout)
NCB *ncbp;
int length;
char *buffer;
unsigned char timeout;
    {
    if (ncbp->NCB_NUM == 0) /*verify path exists*/
        return (-1);
    ncbp->NCB_LENGTH = length - 1;  /*copy length to ncb*/
    ncbp->NCB_BUFFER = buffer + 1;  /*copy buffer address to ncb*/
    ncbp->NCB_STO = timeout;    /*copy timeout to ncb*/
    ncbp->NCB_COMMAND = SEND;   /*send command*/
    ncbp->NCB_RETCODE = ERR_bad_card_num;  /*invalidate return code*/
	 ncbp->NCB_LSN = 0;
	 ncbp->NCB_POST_ADDRESS = NULL;
    exec_netbios(ncbp); /*execute*/
    return (ncbp->NCB_RETCODE);
    }


/*
ncb_cancel()

Cancel an outstanding transaction.
*/

int ncb_cancel(ncbp)
NCB *ncbp;
    {
    if (ncbp->NCB_NUM == 0) /*verify path exists*/
        return (-1);
    ncbp->NCB_COMMAND = CANCEL; /*send command*/
    ncbp->NCB_RETCODE = ERR_bad_card_num;  /*invalidate return code*/
	 ncbp->NCB_LSN = 0;
	 ncbp->NCB_POST_ADDRESS = NULL;
    exec_netbios(ncbp); /*execute*/
    return (ncbp->NCB_RETCODE);
    }

/*
ncb_receive()

Receive a message on the given path.
Note:
ncb_receive() has changed for rev 2.  For rev 1, it would wait until
the response came in, or it timed out.
Now, ncb_receive_wait() does that.
If the response is not yet available when ncb_receive() is called,
it returns immediately with the ERR_timeout status.
This makes it more useful in a multi-tasking environment.

When the operation is complete, the length of the received packet
is contained in NCB_LENGTH.
*/

int ncb_receive(ncbp, buffer)
NCB *ncbp;
char *buffer;
    {
    int    i;

    if (ncbp->NCB_NUM == 0) /*verify path exists*/
        return (-1);
    ncbp->NCB_BUFFER = buffer + 1;  /*copy buffer address to ncb*/
    ncbp->NCB_RTO = 0;  /* not used for RECEIVE */
    ncbp->NCB_COMMAND = RECEIVE;    /* rev 1 driver used RECEIVE receive command*/
    ncbp->NCB_RETCODE = ERR_bad_card_num;  /*invalidate return code*/
	 ncbp->NCB_LSN = 0;
/*	 ncbp->NCB_POST_ADDRESS = NULL;  */
    exec_netbios(ncbp); /*execute*/
    i = 0;
    while( ncbp->NCB_CALLNAME[ 2 + i ] && ( i < 5 ) )
        i++;
    if( i )
        i--;                        /* backup to last non-zero routing byte */
    buffer[ 0 ] = ncbp->NCB_CALLNAME[ 2 + i ];

    return (ncbp->NCB_RETCODE);
    }


/*
ncb_receive_wait()


Receive a message on the given path.
Note: this is the same as the rev 1 receive() function.
See the comments for ncb_receive() as well.
*/

int ncb_receive_wait(ncbp, buffer, timeout)
NCB *ncbp;
char *buffer;
unsigned char timeout;
    {
    int    i;

    if (ncbp->NCB_NUM == 0) /*verify path exists*/
        return (-1);
    ncbp->NCB_BUFFER = buffer + 1;  /*copy buffer address to ncb*/
    ncbp->NCB_RTO = timeout;    /*copy timeout to ncb*/
/*  ncbp->NCB_COMMAND = RECEIVE;    rev 1 driver used RECEIVE receive command*/
    ncbp->NCB_COMMAND = RECEIVE_WAIT;   /*receive command, rev 2 driver*/
    ncbp->NCB_RETCODE = ERR_bad_card_num;  /*invalidate return code*/
	 ncbp->NCB_LSN = 0;
	 ncbp->NCB_POST_ADDRESS = NULL;
    exec_netbios(ncbp); /*execute*/
    i = 0;
    while( ncbp->NCB_CALLNAME[ 2 + i ] && ( i < 5 ) )
        i++;
    if( i )
        i--;                        /* backup to last non-zero routing byte */
    buffer[ 0 ] = ncbp->NCB_CALLNAME[ 2 + i ];
    return (ncbp->NCB_RETCODE);

    }

/*
ncb_open()


Open a path. Return pointer to NCB, or NULL on error.
*/
#ifdef __OS2__
NCB *ncb_open(name, adaptno)
char *name;
int adaptno;
	{
	int i, j, k;
	NCB temp_ncb, *temp_ncbp = &temp_ncb;
	unsigned char temp_path[2];
	unsigned char *ucp;
	unsigned char r[5];
	char *cptr = name;
	unsigned char *ucptr = r;

	if ((adaptno != 0) && (adaptno != 1))
		return (NCB *) NULL;

    /*
     * First, check to see if it's for a Slave path:
     * Example: ncb_open("DS.1.0.0.0.0",0);
     *        open slave data path #1 (1-8) (internal 0x41-0x48), 
     *        lan adapter #0
     *        (for Slave paths, the path number must be explicitly
     *         chosen, but no other routing information is specified )
     */
	if (toupper(name[1]) == 'S') {
		temp_path[0] = toupper(name[3]);
		temp_path[1] = 0;
		if (toupper(name[0]) == 'D')
			temp_ncbp->NCB_NUM = (unsigned char) (0x40 + (atoi(temp_path)));
		else 
			temp_ncbp->NCB_NUM = (unsigned char) (0xC0 + (atoi(temp_path)));

        temp_ncbp->NCB_CALLNAME[0] = name[0];                 /*'D' or 'P'*/
        temp_ncbp->NCB_CALLNAME[1] = name[1];                 /*'S'    'S'*/
        temp_ncbp->NCB_CALLNAME[2] = temp_ncbp->NCB_NUM & 0xf;/* 1      1 */
        temp_ncbp->NCB_CALLNAME[3] = 0;                       /* 0      0 */
        temp_ncbp->NCB_CALLNAME[4] = 0;                       /* 0      0 */
        temp_ncbp->NCB_CALLNAME[5] = 0;                       /* 0      0 */
        temp_ncbp->NCB_CALLNAME[6] = 0;                       /* 0      0 */

		temp_ncbp->NCB_COMMAND = ADD_NAME;
		temp_ncbp->NCB_LANA_NUM = (unsigned char) adaptno;
	 	temp_ncbp->NCB_LSN = 0;
	 	temp_ncbp->NCB_POST_ADDRESS = NULL;
		exec_netbios(temp_ncbp);
		/*
		 * Explanation:
		 * 1) User fills in dummy NCB
		 * 2) user calls add_name, gets assigned a real NCB
		 * 3) driver copies user's values into real NCB
		 * 4) drivers stuffs address of real NCB into NCB_BUFFER entry
		 * 5) back in netlib.c's ncb_open routine, real pointer to NCB is obtained
		 */
		if (temp_ncbp->NCB_RETCODE == 0)
			return ((NCB *) temp_ncbp->NCB_BUFFER);
						/*return pointer to real NCB */
		return ((NCB *) NULL);	/*return null pointer*/
		}
    /*
     * else, it must be for a Master path:
     * Example: ncb_open("DM.13.5.0.0.0",0);
     *        open master data path to node 5 going through router
     *        at node 13,
     *        lan adapter #0
     *        (internal path used for master paths is automatically
     *         selected from those not in use)
     */
	else {
		k = 5;
		j = strlen(name);
		if (j < 2)		/*types limited to 2*/
			return ((NCB *) NULL);
		i = 0;
		while (!(isdigit(*cptr)) && *cptr) {
			cptr++;
			i++;
			}
		*ucptr = '\0';
		while (*cptr && i < j && k != 0) {
			if (isdigit(*cptr)) {
				*ucptr = ((*ucptr) * 10) + *cptr - '0';
				}
			else {
				ucptr++;
				*ucptr = '\0';
				k--;
				}
			cptr++;
			i++;
			}
		ucp = temp_ncbp->NCB_CALLNAME;
		*ucp++ = *name;
		*ucp++ = *(name + 1);	/*fill in path*/
		*ucp++ = r[0];	/*convert to binary*/
		*ucp++ = r[1];
		*ucp++ = r[2];
		*ucp++ = r[3];
		*ucp++ = r[4];	/*fill in routing info*/
		temp_ncbp->NCB_COMMAND = ADD_NAME;
		temp_ncbp->NCB_LANA_NUM = (unsigned char) adaptno;
	 	temp_ncbp->NCB_LSN = 0;
	 	temp_ncbp->NCB_POST_ADDRESS = NULL;
		exec_netbios(temp_ncbp);
		if (temp_ncbp->NCB_RETCODE == 0)
			return ((NCB *) temp_ncbp->NCB_BUFFER);
						/*return pointer to real NCB */
		return ((NCB *) NULL);	/*return null pointer*/
		}
	}

#else /**************** Regular DOS ncb_open start here ***************/

NCB *ncb_open(name, adaptno)
char *name;
int adaptno;
    {
    int i, j, k;
    NCB *ncbp;
    unsigned char temp_path[2];
    unsigned char *ucp;
    unsigned char r[5];
    char *cptr = name;
    unsigned char *ucptr = r;

	if ((adaptno != 0) && (adaptno != 1))
		return (NCB *) NULL;

    /*
     * First, check to see if it's for a Slave path:
     * Example: ncb_open("DS.1.0.0.0.0",0);
     *        open slave data path #1 (1-8) (internal 0x41-0x48), 
     *        lan adapter #0
     *        (for Slave paths, the path number must be explicitly
     *         chosen, but no other routing information is specified )
     */

    if (toupper(name[1]) == 'S') {
        for (i = 0;  i < MAX_NCB;  i++) {   /*look for first open NCB*/
            ncbp = (NCB *) &ncbs[i];
            if (ncbp->NCB_NUM == 0) {   /*use first available*/
                temp_path[0] = toupper(name[3]);
                temp_path[1] = 0;
                if (toupper(name[0]) == 'D')
                    ncbp->NCB_NUM = (unsigned char) (0x40 + (atoi(temp_path)));
                else 
                    ncbp->NCB_NUM = (unsigned char) (0xC0 + (atoi(temp_path)));

                ncbp->NCB_CALLNAME[0] = name[0];                /*'D' or 'P'*/
                ncbp->NCB_CALLNAME[1] = name[1];                /*'S'    'S'*/
                ncbp->NCB_CALLNAME[2] = ncbp->NCB_NUM & 0xf;    /* 1      1 */
                ncbp->NCB_CALLNAME[3] = 0;                      /* 0      0 */
                ncbp->NCB_CALLNAME[4] = 0;                      /* 0      0 */
                ncbp->NCB_CALLNAME[5] = 0;                      /* 0      0 */
                ncbp->NCB_CALLNAME[6] = 0;                      /* 0      0 */
                ncbp->NCB_COMMAND = ADD_NAME;
                ncbp->NCB_LANA_NUM = (unsigned char) adaptno;
                ncbp->NCB_RETCODE = ERR_bad_card_num;  /*invalidate return code*/
                exec_netbios(ncbp);
                if (ncbp->NCB_RETCODE == 0)
                    return (ncbp);  /*return pointer to NCB*/
                return ((NCB *) NULL);  /*return null pointer*/
                }
            }
        return ((NCB *) NULL);  /*return null pointer*/
        }
    /*
     * else, it must be for a Master path:
     * Example: ncb_open("DM.13.5.0.0.0",0);
     *        open master data path to node 5 going through router
     *        at node 13,
     *        lan adapter #0
     *        (internal path used for master paths is automatically
     *         selected from those not in use)
     */
    else {
        k = 5;
        j = strlen(name);
        if (j < 2)      /*types limited to 2*/
            return ((NCB *) NULL);
        i = 0;
        while (!(isdigit(*cptr)) && *cptr) {
            cptr++;
            i++;
            }
        *ucptr = '\0';
        while (*cptr && i < j && k != 0) {
            if (isdigit(*cptr)) {
                *ucptr = ((*ucptr) * 10) + *cptr - '0';
                }
            else {
                ucptr++;
                *ucptr = '\0';
                k--;
                }
            cptr++;
            i++;
            }

        for (i = 0;  i < MAX_NCB;  i++) {   /*look for first open NCB*/
            ncbp = (NCB *) &ncbs[i];
            if (ncbp->NCB_NUM == 0) {   /*use first available*/
                ucp = ncbp->NCB_CALLNAME;
                *ucp++ = *name;
                *ucp++ = *(name + 1);   /*fill in path*/
                *ucp++ = r[0];  /*convert to binary*/
                *ucp++ = r[1];
                *ucp++ = r[2];
                *ucp++ = r[3];
                *ucp++ = r[4];  /*fill in routing info*/
                ncbp->NCB_COMMAND = ADD_NAME;
                ncbp->NCB_LANA_NUM = (unsigned char) adaptno;
                ncbp->NCB_RETCODE = ERR_bad_card_num;  /*invalidate return code*/
                exec_netbios(ncbp);
                if (ncbp->NCB_RETCODE == 0)
                    return (ncbp);  /*return pointer to NCB*/
                return ((NCB *) NULL);  /*return null pointer*/
                }
            }
        return ((NCB *) NULL);  /*return null pointer*/
        }
    }

#endif

/*
ncb_close()


Close a path. Returns 0 if successful.
Checks return, and if ERR_invalid (3) is returned, which means
an outstanding transaction must first be CANCEL'd, then it cancels
the transaction and attempts to close the path again.
*/

int ncb_close(ncbp)
NCB *ncbp;
    {
    ncbp->NCB_COMMAND = DELETE_NAME;
    ncbp->NCB_RETCODE = ERR_bad_card_num;  /*invalidate return code*/
	 ncbp->NCB_LSN = 0;
	 ncbp->NCB_POST_ADDRESS = NULL;
    exec_netbios(ncbp);
    ncbp->NCB_NUM = 0;  /* only if good status */
    return ((int) ncbp->NCB_RETCODE);
    }

/*
ncb_send_datagram()


*/

int ncb_send_datagram(ncbp, length, buffer, timeout, adaptno)
NCB *ncbp;
int length;
char *buffer;
unsigned char timeout;
int adaptno;
    {

    if (length > 64)
        return (ERR_bad_buffer_length);
    ncbp->NCB_LANA_NUM = (unsigned char) adaptno;
    ncbp->NCB_COMMAND = SEND_DATAGRAM;
    ncbp->NCB_BUFFER = buffer;
    ncbp->NCB_STO = timeout;    /*copy timeout to ncb*/
    if (length & 1)
        ncbp->NCB_LENGTH = (length / 2) + 1;
                        /*convert to words*/
    else 
        ncbp->NCB_LENGTH = length / 2;  /*convert to words*/
    ncbp->NCB_RETCODE = ERR_bad_card_num;  /*invalidate return code*/
	 ncbp->NCB_LSN = 0;
	 ncbp->NCB_POST_ADDRESS = NULL;
    exec_netbios(ncbp);
    return ((int) ncbp->NCB_RETCODE);
    }


/*
ncb_receive_datagram()

The length in words is returned in NCB_LENGTH
*/

int ncb_receive_datagram(ncbp, node, buffer, timeout, adaptno)
NCB *ncbp;
int node;
char *buffer;
unsigned char timeout;
int adaptno;
    {
    ncbp->NCB_LANA_NUM = (unsigned char) adaptno;
    ncbp->NCB_COMMAND = RECEIVE_DATAGRAM;   /*get global data*/
    ncbp->NCB_NUM = (unsigned char) node;
    ncbp->NCB_BUFFER = buffer;
    ncbp->NCB_RTO = timeout;    /*copy timeout to ncb*/
    ncbp->NCB_RETCODE = ERR_bad_card_num;  /*invalidate return code*/
	 ncbp->NCB_LSN = 0;
	 ncbp->NCB_POST_ADDRESS = NULL;
    exec_netbios(ncbp);
    return (ncbp->NCB_RETCODE);
    }


/*
ncb_set_slave_login()


Set login status on a slave path.
login_status = 1 = set login status
login_status = 0 = clear login status

The path must have been previously opened.
Closing the path resets the login status to not logged in.
Returns are 0 for success, 0xff for error.
*/

int ncb_set_slave_login(ncbp, login_status)
NCB *ncbp;
unsigned char login_status;
    {
    unsigned char buffer[2];

    if (ncbp->NCB_NUM == 0) /*verify path exists*/
        return (-1);
    ncbp->NCB_BUFFER = buffer;  /*copy buffer address to ncb*/
    ncbp->NCB_BUFFER[0] = login_status;
    ncbp->NCB_COMMAND = SET_SLAVE_LOGIN;    /*send command*/
    ncbp->NCB_RETCODE = ERR_bad_card_num;  /*invalidate return code*/
	 ncbp->NCB_LSN = 0;
	 ncbp->NCB_POST_ADDRESS = NULL;
    exec_netbios(ncbp); /*execute*/
    return (ncbp->NCB_RETCODE);
    }

/*
exec_netbios()



Execute a NetBIOS command via the software interrupt.
*/

void exec_netbios(ncbstruct)
NCB *ncbstruct;
    {
#ifdef __OS2__
	mbp_netbios(ncbstruct);	/* go do it in DLL */
	return;
#else
    int fIsEnhancedWindows;
    union REGS inregs, outregs;
    struct SREGS segregs;
    int result_ax;
    unsigned long far *ivector;

    dv_get_version();   /* see if DESQview is there */
        /*
         * Note, dv_get_version squirrels away in its own variable whether
         * DESQviews there or not, so we don't need to qualify the
         * call to dv_begin_critical() or dv_end_critical()
         */
    dv_begin_critical();    /* begin DESQview critical area */

    inregs.x.bx = FP_OFF(ncbstruct);    /*BX = offset of structure*/
    segregs.es = FP_SEG(ncbstruct); /*ES = segment of pointer*/
/*
 * Note: The default value for sw_interrupt is 0x5c, the traditional
 * value for the NetBIOS s/w interrupt.
 * If you change it, the values in the global variable "sw_interrupt"
 * and the setting for the sa85.sys driver in config.sys must match.
 */
    ivector = (unsigned long far *) (long) (sw_interrupt * 4);
    if ((fIsEnhancedWindows = IsEnhancedWindows()))
      WinEnterCriticalSection();
    if(*ivector)                        /*look for non-null interrupt vector*/
        result_ax = int86x(sw_interrupt,    /*software interrupt number*/
            &inregs,    /*array of registers input*/
            &outregs,   /*array of registers output*/
            &segregs);      /*image of segment registers*/
  if (fIsEnhancedWindows)
      WinEndCriticalSection();
    dv_end_critical();  /* end DESQview critical area */
#endif
    }


