/*name test4b.c*/


/*
This test program uses the MPHOST.SYS device driver to read 125 holding
registers from the MODBUS slave which is found at the given node number.

In order to use this program, enter the following command:

    A>TEST4B <1st slave node> <2nd slave node>

    eg. TEST4B 12 14

where "12" is the first slave node we want to read from, and 14 is the second
slave node we want to read from.
*/


/*
include


*/
#define INCL_SUB
#include <os2.h>

#include <stdio.h>
#include <dos.h>
#include <process.h>
#include <string.h>
#include <conio.h>
#include "netbios.h"
#include "netlib.h"


/*
prototypes

*/
int main(int argc, char *argv[] );


/*
global variables


*/
char mbuffer[256];
char path1[ 80 ];
char path2[ 80 ];

#define LENGTH 81       /* length of string buffer */
#define HANDLE 0        /* vio handle always 0 */

/*
main()


*/

int main( argc, argv )
int argc;
char *argv[];
    {
    NCB *nd1, *nd2;
    int i,j,ret_val;

    char buffer[LENGTH];
    unsigned char attr = 0x70;  /* reverse video attribute */
    long count = 0L;
    unsigned Cell = 0x0720; /* for scrolling */


    VioScrollUp(0,0,-1,-1,-1,(char far *)&Cell,0); /* clear the screen */

    sprintf(buffer,"            << TEST4B >>                                    ");
    VioWrtCharStrAtt(buffer, strlen(buffer), 2, 3, &attr, HANDLE);
    sprintf(buffer,"AEG Modicon MODBUS Command/Response test for Modbus+ OS/2   ");
    VioWrtCharStrAtt(buffer, strlen(buffer), 3, 3, &attr, HANDLE);
    sprintf(buffer,"Makes Modbus requests from 2 SA85 boards at the same time.  ");
    VioWrtCharStrAtt(buffer, strlen(buffer), 4, 3, &attr, HANDLE);

    if( argc < 3 ) {
        sprintf(buffer, "Usage is: A>TEST4B <first slave node> <second slave node>   ");
        VioWrtCharStrAtt(buffer, strlen(buffer), 5, 3, &attr, HANDLE);
        exit( 1 );
    }
    ret_val = sscanf( argv[ 1 ], "%d", &i );
    ret_val += sscanf( argv[ 2 ], "%d", &j );
    if( ret_val != 2 || i < 1 || i > 64 ) {
        sprintf(buffer, "Node numbers must be in the range 1 to 64. " );
        VioWrtCharStrAtt(buffer, strlen(buffer), 5, 3, &attr, HANDLE);
        exit( 1 );
    }
    sprintf( path1, "DM.%d.0.0.0.0", i );
    sprintf( path2, "DM.%d.0.0.0.0", j );

    if ((nd1 = ncb_open( path1, 0 )) == NULL) {
        sprintf(buffer,"Unable to open first DATA MASTER path.");
        VioWrtCharStrAtt(buffer, strlen(buffer), 5, 3, &attr, HANDLE);
        exit(1);
        }
    if ((nd2 = ncb_open( path2, 1 )) == NULL) {
        sprintf(buffer,"Unable to open second DATA MASTER path.");
        VioWrtCharStrAtt(buffer, strlen(buffer), 5, 3, &attr, HANDLE);
        exit(1);
        }
    sprintf(buffer,"First board,  Path %02X opened                                ", nd1->NCB_NUM);
    VioWrtCharStrAtt(buffer, strlen(buffer), 5, 3, &attr, HANDLE);
    sprintf(buffer,"Routing info: %c%c.%d.%d.%d.%d.%d                                  ",
        nd1->NCB_CALLNAME[0],
        nd1->NCB_CALLNAME[1],
        nd1->NCB_CALLNAME[2],
        nd1->NCB_CALLNAME[3],
        nd1->NCB_CALLNAME[4],
        nd1->NCB_CALLNAME[5],
        nd1->NCB_CALLNAME[6]);
    VioWrtCharStrAtt(buffer, strlen(buffer), 6, 3, &attr, HANDLE);

    sprintf(buffer,"Second board, Path %02X opened                                ", nd2->NCB_NUM);
    VioWrtCharStrAtt(buffer, strlen(buffer), 7, 3, &attr, HANDLE);
    sprintf(buffer,"Routing info: %c%c.%d.%d.%d.%d.%d                                  ",
        nd2->NCB_CALLNAME[0],
        nd2->NCB_CALLNAME[1],
        nd2->NCB_CALLNAME[2],
        nd2->NCB_CALLNAME[3],
        nd2->NCB_CALLNAME[4],
        nd2->NCB_CALLNAME[5],
        nd2->NCB_CALLNAME[6]);
    VioWrtCharStrAtt(buffer, strlen(buffer), 8, 3, &attr, HANDLE);

    for (;  ;) {
        mbuffer[0] = 0x5;   /*slave address*/
        mbuffer[1] = 0x3;   /*command*/
        mbuffer[2] = 0x0;   /*offset high*/
        mbuffer[3] = 0x0;   /*offset low*/
        mbuffer[4] = 0x0;   /*reg count high*/
        mbuffer[5] = 125;   /* reg count low*/
        if (ncb_send(nd1, 6, mbuffer, 10) != 0) {   /*send first command*/
                        /*send the command*/
            sprintf(buffer,"Send error: %d.", nd1->NCB_RETCODE);
            VioWrtCharStrAtt(buffer, strlen(buffer), 9, 3, &attr, HANDLE);
            ret_val = ncb_close(nd1);   /*close the path*/
            ret_val = ncb_close(nd2);   /*close the path*/
            exit(1);
            }

        mbuffer[0] = 0x5;   /*slave address*/
        mbuffer[1] = 0x3;   /*command*/
        mbuffer[2] = 0x0;   /*offset high*/
        mbuffer[3] = 0x0;   /*offset low*/
        mbuffer[4] = 0x0;   /*reg count high*/
        mbuffer[5] = 125;   /* reg count low*/
        if (ncb_send(nd2, 6, mbuffer, 10) != 0) {   /*send second command*/
                        /*send the command*/
            sprintf(buffer,"Send error: %d.", nd2->NCB_RETCODE);
            VioWrtCharStrAtt(buffer, strlen(buffer), 9, 3, &attr, HANDLE);
            ret_val = ncb_close(nd1);   /*close the path*/
            ret_val = ncb_close(nd2);   /*close the path*/
            exit(1);
            }

        if (ncb_receive_wait(nd1, mbuffer, 10) != 0) {
                        /*try to receive*/
            sprintf(buffer,"Receive error: %d.", nd1->NCB_RETCODE);
            VioWrtCharStrAtt(buffer, strlen(buffer), 9, 3, &attr, HANDLE);
            }

        if (ncb_receive_wait(nd2, mbuffer, 10) != 0) {
                        /*try to receive*/
            sprintf(buffer,"Receive error: %d.", nd2->NCB_RETCODE);
            VioWrtCharStrAtt(buffer, strlen(buffer), 9, 3, &attr, HANDLE);
            }
        count+=2;
        sprintf(buffer, "# of transactions through BOTH boards: %ld ", count);
        VioWrtCharStrAtt(buffer, strlen(buffer), 9, 3, &attr, HANDLE);
        if (kbhit()) {
            sprintf(buffer, "<< TEST4B TERMINATED NORMALLY >>");
            VioWrtCharStrAtt(buffer, strlen(buffer), 10, 3, &attr, HANDLE);
            getch();
            ncb_close(nd1);     /*close the path*/
            ncb_close(nd2);     /*close the path*/
            VioSetCurPos(11,1,HANDLE);
            exit(0);
            }
        }
    ncb_close(nd1);     /*close the path*/
    ncb_close(nd2);     /*close the path*/
    exit(0);
    return( 0 );
    }
