/* ************************************************************
 * Modbus Plus Host Device Driver Interface
 **************************************************************/


/* ************************************************************
 * COPYRIGHT (C) 1991-1992, MODICON INC., ALL RIGHTS RESERVED.
 * NO PART OF THIS DOCUMENT MAY BE REPRODUCED IN ANY FORM WITHOUT THE
 * EXPRESSED WRITTEN PERMISSION OF MODICON INC.
 *
 *
 * program name:   PG-MBPL-500
 * revision:       -
 * version:        2.00
 * eco number:     16491
 * release date:   10/16/92
 *
 *
 *
 * R E V I S I O N   H I S T O R Y
 *
 * DATE      NAME            CHANGES
 *       
 * 09/Oct/96 Papadopoulos    Added the valid data fields to the DDSTATUS struct.
 **************************************************************/

#define CAT_IOCTLIN     0x80
#define CAT_IOCTLOUT    0x81
#define CAT_IOCTLRESET  0x82
#define CAT_IOCTLMAINT  0x83

#define DCMD_DEVRESET   0
#define DCMD_DEVSTATUS  0
#define DCMD_OPENPATH   0
#define DCMD_CLOSEPATH  1
#define DCMD_DEVCMD     2

#define MCMD_KICK       0
#define MCMD_GSRSEMHND  1
#define MCMD_RDSLAVE    2
#define MCMD_RFSLAVE    3

#define OPC_GETSRVCREQ  0x0
#define OPC_PUTCOMMAND  0x1
#define OPC_GETCOMMAND  0x2
#define OPC_PUTRESPNSE  0x3
#define OPC_GETRESPNSE  0x4
#define OPC_ABORTTRANS  0x5
#define OPC_CNFGSTATUS  0x6
#define OPC_IFCDIAGTST  0x7
#define OPC_SFTWRRESET  0x8
#define OPC_PUTGLBDATA  0x9
#define OPC_GETGLBDATA  0xa

#define CMP_IOCMP       0x0001
#define CMP_IOERR       0x0002
#define CMP_IOCRASH		0x0004
#define CMP_IOTMOUT		0x0008
#define CMP_IOQUE       0x0010
#define CMP_IOBLOCKED	0x0020
#define CMP_IOSTRT		0x0040
#define CMP_IOTTACT		0x0100
#define CMP_IOPEND		0x0200

#define ERR_NOERR       0x0000		/* no errors                             */
#define ERR_BADCMD		0x0001		/* bad command passed to driver          */
#define ERR_BADPATH		0x0002		/* bad path passed to driver             */
#define ERR_PATHINUSE	0x0003		/* path sent is in use                   */
#define ERR_PATHCLOSED	0x0004		/* path is already closed                */
#define ERR_BADOPCODE	0x0005		/* bad opcode with devcmd                */
#define ERR_BADBUFF		0x0006		/* bad buffer passed to driver           */
#define ERR_ABORTED		0x0007		/* path was aborted                      */
#define ERR_BADMAP		0x0008		/* bad address to devhlp mapping         */
#define ERR_TIMEOUT		0x0009		/* aborted by timeout                    */
#define ERR_IFCCRASH    0x000A		/* interface crashed                     */

#define BUFFSIZE        288         /* MB+ transaction buffer size           */

/*===!!!=====================================================================*/
/* the PDD is a 16bit application. This means every pointer that is passed   */
/* between the driver and the 32bit DLL must be converted by _Seg16.         */
/* The only exeption of this rule is the pointer in the uservar field that   */
/* is returned by the driver upon the DCMD_OPENPATH command. It is a flat    */
/* 32bit pointer.                                                            */
/*===!!!=====================================================================*/
#define DD_SEGMENT      _Seg16      


typedef struct ddreq
{
	struct   ddreq * DD_SEGMENT qlink;
	unsigned short              reqver;
	unsigned short              cmpflg;
	unsigned short              status;
	PFN              DD_SEGMENT cmplrt; /* the completion routine             */
	void           *            uservar;/* this returns 0:32 ptr to paht buf  */
	unsigned long               timeout;
	unsigned char               cmd;
	unsigned char               pathid;
	unsigned short              buffsize;
	unsigned char  * DD_SEGMENT pbuffer;
}  _Packed volatile DDREQ;          /* do not allow gaps in this struct      */

typedef DDREQ * PDDREQ;


typedef struct ddstatus
{
	unsigned short  ssize;
	unsigned short  sver;
	unsigned char   irqchoice;
	unsigned short  dpsegment;
	unsigned char   lannumber;
	unsigned short  ddopencount;
	unsigned char   crashcode;
	unsigned char   lastopcode;
	unsigned char   nodeaddress;
	unsigned char   enhancedpoll;
	unsigned char   pmcsmap;
	unsigned char   pslrmap;
	unsigned char   dmopen;
	unsigned char   dsopen;
	unsigned char   pmopen;
	unsigned char   psopen;
	unsigned char   dmgrmap;
	unsigned char   dsgcmap;
	unsigned char   pmgrmap;
	unsigned char   psgcmap;
	unsigned char   gdpmap[ 8 ];
	unsigned char   dmgrdef;
	unsigned char   dsgcdef;
	unsigned char   pmgrdef;
	unsigned char   psgcdef;
	unsigned char   dmrtfpnd;
	unsigned char   dsrtfpnd;
	unsigned char   pmrtfpnd;
	unsigned char   psrtfpnd;
	unsigned char   dmvalidd;              /* data master valid data */
	unsigned char   dsvalidd;              /* data slave valid data	   */
	unsigned char   pmvalidd;              /* program master valid data */
	unsigned char   psvalidd;              /* program slave valid data  */
} _Packed volatile DDSTATUS;        /* do not allow gaps in this struct      */


